﻿using RocketLog;
using UnityEngine;

#if ROC_GM_APPSFLYER
using RocketGMAppsFlyer;
#endif
#if ROC_FACEBOOK
using RocketFacebook;
#endif
#if ROC_FIREBASE
using Firebase;
#endif

namespace RocketRevenueTracker
{
    [System.Serializable]
    public class RocketRevenueTracker
    {
        private bool _isSendingInProgress = false;
        [SerializeField]
        [Tooltip("In dollars (float)\nExample: 0.10 dollar means 10 cents")]
        private float _revenueThreshold = 0.10f;
        [SerializeField]
        [Tooltip("In cents with prefix _ (string)\nExample: _10 means 10 cents")]
        private string _eventSuffix = "_10";
        
        private readonly string _eventPrefix = "revenue_reached";
        private readonly bool _isDuplicateEventLoggingDisabled;

#if ROC_GM_APPSFLYER
        public static bool SendAppsflyer = true;
#endif
#if ROC_FACEBOOK
        public static bool SendFacebook = true;
#endif
#if ROC_FIREBASE
        public static bool SendFirebase = true;
#endif

        private RocLog _log;

        public float RevenueAmount
        {
            get => PlayerPrefs.GetFloat($"RocGMRevenueTrackerAmount{_eventSuffix}", 0f);
            set => PlayerPrefs.SetFloat($"RocGMRevenueTrackerAmount{_eventSuffix}", value);
        }

        private int IsRevenueSent
        {
            get => PlayerPrefs.GetInt($"RevenueSent{_eventSuffix}", 0);
            set => PlayerPrefs.SetInt($"RevenueSent{_eventSuffix}", value);
        }
        
        public RocketRevenueTracker(float threshold, string suffix, bool isDuplicateEventLoggingDisabled)
        {
            _revenueThreshold = threshold;
            _eventSuffix = suffix;
            _isDuplicateEventLoggingDisabled = isDuplicateEventLoggingDisabled;
            _log = new RocLog(nameof(RocketRevenueTrackerController), DebugLevels.Debug);
        }

        public void TrySendRevenue(float revenueAmount)
        {
            if (IsRevenueSent == 1 && _isDuplicateEventLoggingDisabled) return;

            if (RevenueAmount < _revenueThreshold || _isSendingInProgress)
                return;
            
            _isSendingInProgress = true;
            
            var eventName = $"{_eventPrefix}{_eventSuffix}";
            _log.Debug($"Sending revenue event: {eventName}");
            
#if ROC_GM_APPSFLYER
            if (SendAppsflyer && RocGMAppsFlyerController.Instance != null)
                RocGMAppsFlyerController.Instance.Report(eventName);
#endif
#if ROC_FACEBOOK
            if (SendFacebook)
                RocFacebookController.SendCustomEvent(eventName);
#endif
#if ROC_FIREBASE
            if (SendFirebase)
                RocFirebaseController.SendCustomEvent(eventName);
#endif
            
            RevenueAmount -= revenueAmount;
            if (_isDuplicateEventLoggingDisabled) IsRevenueSent = 1;

            _isSendingInProgress = false;
        }
    }
}